"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.traverseNodes = exports.getNodes = exports.parseYAML = void 0;
const shared_1 = require("@intlify/shared");
const yaml_eslint_parser_1 = require("yaml-eslint-parser");
function parseYAML(code, options = {}) {
    const parserOptions = Object.assign({
        filePath: '<input>',
        ecmaVersion: 2019
    }, options, {
        loc: true,
        range: true,
        raw: true,
        tokens: true,
        comment: true,
        eslintVisitorKeys: true,
        eslintScopeManager: true
    });
    return yaml_eslint_parser_1.parseForESLint(code, parserOptions).ast;
}
exports.parseYAML = parseYAML;
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function fallbackKeysFilter(key) {
    let value = null;
    return (key !== 'comments' &&
        key !== 'sourceType' &&
        key !== 'loc' &&
        key !== 'parent' &&
        key !== 'range' &&
        key !== 'tokens' &&
        (value = this[key]) !== null &&
        shared_1.isObject(value) &&
        (shared_1.isString(value.type) || Array.isArray(value)));
}
function getFallbackKeys(node) {
    return Object.keys(node).filter(fallbackKeysFilter, node);
}
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function isNode(x) {
    return x !== null && shared_1.isObject(x) && shared_1.isString(x.type);
}
// eslint-disable-next-line @typescript-eslint/no-explicit-any, @typescript-eslint/explicit-module-boundary-types
function* getNodes(node, key) {
    const child = node[key];
    if (Array.isArray(child)) {
        for (const c of child) {
            if (isNode(c)) {
                yield c;
            }
        }
    }
    else if (isNode(child)) {
        yield child;
    }
}
exports.getNodes = getNodes;
function getKeys(node, visitorKeys) {
    const keys = (visitorKeys || yaml_eslint_parser_1.VisitorKeys)[node.type] || getFallbackKeys(node);
    return keys.filter((key) => !getNodes(node, key).next().done);
}
function traverse(node, parent, visitor) {
    visitor.enterNode(node, parent);
    const keys = getKeys(node, visitor.visitorKeys);
    for (const key of keys) {
        for (const child of getNodes(node, key)) {
            traverse(child, node, visitor);
        }
    }
    visitor.leaveNode(node, parent);
}
function traverseNodes(node, visitor) {
    traverse(node, null, visitor);
}
exports.traverseNodes = traverseNodes;
